@@ -11,7 +11,7 @@ use strict;
 @ISA = qw(Verilog::Netlist::Net::Struct
 	Verilog::Netlist::Subclass);
 
-$VERSION = '3.430';
+$VERSION = '3.440';
 
 my %_Type_Widths = (
     'bit'	=> 1,
@@ -190,7 +190,10 @@ sub lint {
     }
     elsif ($self->_used_out()) {
 	if ($self->_used_out()>1
-	    && !$self->array()) {   # if an array, different outputs might hit different bits
+	    # if an array, different outputs might hit different bits
+	    && !$self->array()
+	    # if vector, warn only if # of usages is higher than # of bits in vector
+	    && (abs($self->msb() - $self->lsb()) + 1) < $self->_used_out()) {
 	    $self->warn("Signal has multiple drivers (",
 			$self->_used_out(),"): ",$self->name(), "\n");
 	    $self->dump_drivers(8);
@@ -261,14 +264,17 @@ sub dump_drivers {
     }
     foreach my $cell ($self->module->cells_sorted) {
 	foreach my $pin ($cell->pins_sorted) {
-	    if ($pin->port && $pin->net && $pin->net == $self) {
-		print " "x$indent,"  Pin:  ",$cell->name,".",$pin->name
-		    ,"  ",$pin->port->direction,"\n";
-	    }
-	    elsif ($pin->net && $self->name eq $pin->net->name) {
-		warn "%Warning: Internal net name duplicate: ".$cell->name."  ".$self->name."\n"
-		    .$self->comment."  ".$pin->net->comment."\n"
-		    ."$self  ".$pin->net->name."\n";
+	    foreach my $net ($pin->nets) {
+		next unless defined $net->{net};
+		if ($pin->port && $net->{net} == $self) {
+		    print " "x$indent,"  Pin:  ",$cell->name,".",$pin->name
+			,"  ",$pin->port->direction,"\n";
+		}
+		elsif ($self->name eq $net->{net}->name) {
+		    warn "%Warning: Internal net name duplicate: ".$cell->name."  ".$self->name."\n"
+			.$self->comment."  ".$net->{net}->comment."\n"
+			."$self  ".$net->{net}->name."\n";
+		}
 	    }
 	}
     }
